@@ -1,3 +1,21 @@
+/*
+ * Copyright 2008-2013 Various Authors
+ * Copyright 2004 Timo Hirvonen
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ */
+
 #include "buffer.h"
 #include "xmalloc.h"
 #include "locking.h"
@@ -39,6 +57,11 @@ void buffer_init(void)
 	buffer_reset();
 }
 
+void buffer_free(void)
+{
+	free(buffer_chunks);
+}
+
 /*
  * @pos: returned pointer to available data
  *
@@ -90,7 +113,7 @@ void buffer_consume(int count)
 {
 	struct chunk *c;
 
-	BUG_ON(count <= 0);
+	BUG_ON(count < 0);
 	cmus_mutex_lock(&buffer_mutex);
 	c = &buffer_chunks[buffer_ridx];
 	BUG_ON(!c->filled);
