@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2011 Various Authors
+ * Copyright 2008-2013 Various Authors
  * Copyright 2004 Timo Hirvonen
  *
  * This program is free software; you can redistribute it and/or
@@ -16,8 +16,8 @@
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef CONVERT_H
-#define CONVERT_H
+#ifndef CMUS_CONVERT_H
+#define CMUS_CONVERT_H
 
 #include <sys/types.h> /* ssize_t */
 
@@ -28,4 +28,6 @@ ssize_t convert(const char *inbuf, ssize_t inbuf_size,
 
 int utf8_encode(const char *inbuf, const char *encoding, char **outbuf);
 
+char *to_utf8(const char *str, const char *enc);
+
 #endif
