@@ -1,6 +1,7 @@
-/* 
+/*
+ * Copyright 2008-2013 Various Authors
  * Copyright 2004 Timo Hirvonen
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -10,15 +11,13 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef _HISTORY_H
-#define _HISTORY_H
+#ifndef CMUS_HISTORY_H
+#define CMUS_HISTORY_H
 
 #include "list.h"
 
@@ -32,6 +31,7 @@ struct history {
 
 void history_load(struct history *history, char *filename, int max_lines);
 void history_save(struct history *history);
+void history_free(struct history *history);
 void history_add_line(struct history *history, const char *line);
 void history_reset_search(struct history *history);
 const char *history_search_forward(struct history *history, const char *text);
