@@ -44,6 +44,9 @@ struct playlist {
 	struct simple_track *cur_track;
 };
 
+char *pl_resume_name;
+unsigned long pl_resume_row;
+
 static struct playlist *pl_visible; /* never NULL */
 static struct playlist *pl_marked; /* never NULL */
 struct window *pl_list_win;
@@ -118,7 +121,7 @@ static int pl_search_get_next(struct iter *iter)
 	return pl_search_get_generic(iter, list_next, simple_track_get_next);
 }
 
-static int pl_search_get_current(void *data, struct iter *iter)
+static int pl_search_get_current(void *data, struct iter *iter, enum search_direction dir)
 {
 	window_get_sel(pl_editable_shared.win, iter);
 	iter->data2 = &pl_visible->node;
@@ -211,9 +214,29 @@ static void pl_add_track(struct playlist *pl, struct track_info *ti)
 	editable_add(&pl->editable, track);
 }
 
+static void pl_loaded(struct playlist *pl)
+{
+	if (!pl_resume_name || strcmp(pl->name, pl_resume_name) != 0
+			|| pl_resume_row >= pl->editable.nr_tracks)
+		return;
+
+	struct list_head *li = pl->editable.head.next;
+	for (int i = 0; i < pl_resume_row; i++)
+		li = li->next;
+	pl_playing_track = to_simple_track(li);
+	pl_playing = pl;
+	pl_select_playing_track();
+
+	free(pl_resume_name);
+	pl_resume_name = NULL;
+}
+
 static void pl_add_cb(struct track_info *ti, void *opaque)
 {
-	pl_add_track(opaque, ti);
+	if (ti)
+		pl_add_track(opaque, ti);
+	else
+		pl_loaded(opaque);
 }
 
 int pl_add_file_to_marked_pl(const char *file)
@@ -276,7 +299,7 @@ static void pl_load_all(void)
 
 static void pl_create_default(void)
 {
-	struct playlist *pl = pl_new("default");
+	struct playlist *pl = pl_new("Default");
 	list_add_tail(&pl->node, &pl_head);
 }
 
@@ -419,17 +442,10 @@ static void pl_save_all(void)
 		pl_save_one(pl);
 }
 
-static void pl_delete_selected_pl(void)
+static void pl_delete(struct playlist *pl)
 {
-	if (list_len(&pl_head) == 1) {
-		error_msg("cannot delete the last playlist");
-		return;
-	}
-
-	if (yes_no_query("Delete selected playlist? [y/N]") != UI_QUERY_ANSWER_YES)
-		return;
-
-	struct playlist *pl = pl_visible;
+	if (list_len(&pl_head) == 1)
+		pl_create_default();
 
 	struct iter iter;
 	pl_to_iter(pl, &iter);
@@ -459,6 +475,38 @@ static void pl_delete_selected_pl(void)
 	job_schedule_pl_delete(pdd);
 }
 
+static void pl_delete_selected_pl(void)
+{
+	if (yes_no_query("Delete selected playlist? [y/N]") != UI_QUERY_ANSWER_YES)
+		return;
+
+	pl_delete(pl_visible);
+}
+
+void pl_delete_by_name(char *name)
+{
+	struct playlist *pl;
+	list_for_each_entry(pl, &pl_head, node) {
+		if (strcmp(pl->name, name) == 0) {
+			pl_delete(pl);
+			return;
+		}
+	}
+	error_msg("couldn't find a playlist named '%s' to delete", name);
+}
+
+void pl_delete_all(void)
+{
+	struct playlist *pl;
+	struct playlist *temp;
+	list_for_each_entry_safe(pl, temp, &pl_head, node) {
+		if (list_len(&pl_head) == 1)
+			break;
+		pl_delete(pl);
+	}
+	pl_delete(pl);
+}
+
 static void pl_mark_selected_pl(void)
 {
 	pl_marked = pl_visible;
@@ -574,6 +622,12 @@ void pl_init(void)
 	pl_searchable = searchable_new(NULL, &iter, &pl_searchable_ops);
 }
 
+void pl_init_options(void)
+{
+	if (auto_hide_playlists_panel)
+		pl_cursor_in_track_window = 1;
+}
+
 void pl_exit(void)
 {
 	pl_save_all();
@@ -795,7 +849,7 @@ void pl_invert_marks(void)
 void pl_mark(char *arg)
 {
 	pl_tw_only("mark")
-		editable_invert_marks(&pl_visible->editable);
+		editable_mark(&pl_visible->editable, arg);
 }
 
 void pl_unmark(void)
@@ -864,6 +918,16 @@ void pl_set_nr_rows(int h)
 	window_set_nr_rows(pl_editable_shared.win, h);
 }
 
+bool pl_show_panel(void)
+{
+	return !auto_hide_playlists_panel || !pl_cursor_in_track_window;
+}
+
+char *pl_visible_get_name(void)
+{
+	return pl_visible->name;
+}
+
 unsigned int pl_visible_total_time(void)
 {
 	return pl_visible->editable.total_time;
@@ -945,3 +1009,24 @@ void pl_set_marked_pl_by_name(const char *name)
 		}
 	}
 }
+
+const char *pl_playing_pl_name(void)
+{
+	return pl_playing ? pl_playing->name : NULL;
+}
+
+int pl_playing_pl_row(void)
+{
+	struct list_head *li;
+	int n = 0;
+
+	if (!pl_playing_track)
+		return n;
+
+	list_for_each(li, &pl_playing->editable.head) {
+		if (li == &pl_playing_track->node)
+			break;
+		n++;
+	}
+	return n;
+}
