@@ -1,6 +1,7 @@
-/* 
+/*
+ * Copyright 2008-2013 Various Authors
  * Copyright 2005 Timo Hirvonen
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -10,11 +11,9 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "spawn.h"
@@ -27,7 +26,7 @@
 #include <fcntl.h>
 #include <errno.h>
 
-int spawn(char *argv[], int *status)
+int spawn(char *argv[], int *status, int do_wait)
 {
 	pid_t pid;
 	int err_pipe[2];
@@ -41,7 +40,22 @@ int spawn(char *argv[], int *status)
 		return -1;
 	} else if (pid == 0) {
 		/* child */
-		int dev_null, err;
+		int dev_null, err, i;
+
+		/* create grandchild and exit child to avoid zombie processes */
+		if (!do_wait) {
+			switch (fork()) {
+			case 0:
+				/* grandchild */
+				break;
+			case -1:
+				/* error */
+				_exit(127);
+			default:
+				/* parent of grandchild */
+				_exit(0);
+			}
+		}
 
 		close(err_pipe[0]);
 		fcntl(err_pipe[1], F_SETFD, FD_CLOEXEC);
@@ -56,6 +70,10 @@ int spawn(char *argv[], int *status)
 		/* not interactive, close stdin */
 		close(0);
 
+		/* close unused fds */
+		for (i = 3; i < 30; i++)
+			close(i);
+
 		execvp(argv[0], argv);
 
 		/* error */
@@ -64,13 +82,15 @@ int spawn(char *argv[], int *status)
 		exit(1);
 	} else {
 		/* parent */
-		int rc, errno_save, child_errno;
+		int rc, errno_save, child_errno, tmp;
 
 		close(err_pipe[1]);
 		rc = read_all(err_pipe[0], &child_errno, sizeof(int));
 		errno_save = errno;
 		close(err_pipe[0]);
 
+		if (!do_wait)
+			status = &tmp;
 		waitpid(pid, status, 0);
 
 		if (rc == -1) {
