@@ -1,6 +1,7 @@
-/* 
+/*
+ * Copyright 2008-2013 Various Authors
  * Copyright 2004-2005 Timo Hirvonen
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -10,19 +11,19 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef _TABEXP_FILE_H
-#define _TABEXP_FILE_H
+#ifndef CMUS_TABEXP_FILE_H
+#define CMUS_TABEXP_FILE_H
 
 #include <sys/stat.h>
 
 void expand_files_and_dirs(const char *src,
 		int (*filter)(const char *name, const struct stat *s));
+void expand_env_path(const char *src,
+		int (*filter)(const char *name, const struct stat *s));
 
 #endif
