@@ -405,7 +405,7 @@ static int tree_track_get_next(struct iter *iter)
 }
 
 /* search (tree) {{{ */
-static int tree_search_get_current(void *data, struct iter *iter)
+static int tree_search_get_current(void *data, struct iter *iter, enum search_direction dir)
 {
 	struct artist *artist;
 	struct album *album;
@@ -416,16 +416,38 @@ static int tree_search_get_current(void *data, struct iter *iter)
 		return 0;
 	if (window_get_sel(lib_track_win, &tmpiter)) {
 		track = iter_to_tree_track(&tmpiter);
+		if (search_restricted) {
+			/**
+			 * tree_search_get_next/tree_search_get_prev only return
+			 * the first/last (for forward/backwards) track node for
+			 * restricted search, so we need to make this match so
+			 * do_u_search's check for if we've gone back to the
+			 * starting iter eventually succeeds
+			 */
+			if (dir == SEARCH_FORWARD) {
+				track = to_tree_track(rb_first(&track->album->track_root));
+			} else {
+				track = to_tree_track(rb_last(&track->album->track_root));
+			}
+		}
 		tree_search_track_to_iter(track, iter);
 		return 1;
 	}
 
-	/* artist not expanded. track_win is empty
-	 * set tmp to the first track of the selected artist */
+	/* artist not expanded. track_win is empty set tmp to the first/last
+	 * track of the selected artist for forwards/backwards respectively */
 	window_get_sel(lib_tree_win, &tmpiter);
 	artist = iter_to_artist(&tmpiter);
 	album = to_album(rb_first(&artist->album_root));
-	track = to_tree_track(rb_first(&album->track_root));
+	if (search_restricted) {
+		if (dir == SEARCH_FORWARD) {
+			track = to_tree_track(rb_first(&album->track_root));
+		} else {
+			track = to_tree_track(rb_last(&album->track_root));
+		}
+	} else {
+		track = to_tree_track(rb_first(&album->track_root));
+	}
 	tree_search_track_to_iter(track, iter);
 	return 1;
 }
@@ -828,14 +850,11 @@ static void add_album(struct album *album)
 	struct rb_node **new = &(album->artist->album_root.rb_node), *parent = NULL;
 	struct album *found;
 
-	/*
-	 * Sort regular albums by date, but sort compilations
-	 * alphabetically.
-	 */
-	found = do_find_album(album,
-			      album->artist->is_compilation ? special_album_cmp
-							    : special_album_cmp_date,
-			      &new, &parent);
+	if (sort_albums_by_name || album->artist->is_compilation)
+		found = do_find_album(album, special_album_cmp, &new, &parent);
+	else
+		found = do_find_album(album, special_album_cmp_date, &new, &parent);
+
 	if (!found) {
 		rb_link_node(&album->tree_node, parent, new);
 		rb_insert_color(&album->tree_node, &album->artist->album_root);
@@ -872,7 +891,7 @@ static void album_add_track(struct album *album, struct tree_track *track)
 	rb_insert_color(&track->tree_node, &album->track_root);
 }
 
-static const char *tree_artist_name(const struct track_info* ti)
+const char *tree_artist_name(const struct track_info* ti)
 {
 	const char *val = ti->albumartist;
 
@@ -884,7 +903,7 @@ static const char *tree_artist_name(const struct track_info* ti)
 	return val;
 }
 
-static const char *tree_album_name(const struct track_info* ti)
+const char *tree_album_name(const struct track_info* ti)
 {
 	const char *val = ti->album;
 
@@ -1240,6 +1259,8 @@ void tree_sort_artists(void (*add_album_cb)(struct album *),
 			struct rb_node *t_node, *t_tmp;
 			struct album *album = to_album(l_node);
 
+			remove_album(album);
+			add_album(album);
 			rb_for_each_safe(t_node, t_tmp, &album->track_root) {
 				struct tree_track *track = to_tree_track(t_node);
 
