@@ -51,6 +51,7 @@
 #include "mixer.h"
 #include "mpris.h"
 #include "locking.h"
+#include "pl_env.h"
 #ifdef HAVE_CONFIG
 #include "config/curses.h"
 #include "config/iconv.h"
@@ -107,18 +108,15 @@ int using_utf8 = 0;
 static char *lib_autosave_filename;
 static char *play_queue_autosave_filename;
 
-#define PRINT_BUFFER_SZ 4096
-/* one character can take up to 4 bytes in UTF-8 */
-#define PRINT_BUFFER_MAX_WIDTH (PRINT_BUFFER_SZ / 4 - 1)
+static GBUF(print_buffer);
 
-static char print_buffer[PRINT_BUFFER_SZ];
 /* destination buffer for utf8_encode_to_buf and utf8_decode */
-static char conv_buffer[PRINT_BUFFER_SZ];
+static char conv_buffer[4096];
 
 /* shown error message and time stamp
  * error is cleared if it is older than 3s and key was pressed
  */
-static char error_buf[PRINT_BUFFER_SZ];
+static GBUF(error_buf);
 static time_t error_time = 0;
 /* info messages are displayed in different color */
 static int msg_is_error;
@@ -146,11 +144,14 @@ static int win_active = 1;
 static int show_cursor;
 static int cursor_x;
 static int cursor_y;
+static int cmdline_cursor_x;
 
 static const int default_esc_delay = 25;
 
 static char *title_buf = NULL;
 
+static int in_bracketed_paste = 0;
+
 enum {
 	CURSED_WIN,
 	CURSED_WIN_CUR,
@@ -167,11 +168,12 @@ enum {
 	CURSED_COMMANDLINE,
 	CURSED_STATUSLINE,
 
+	CURSED_STATUSLINE_PROGRESS,
 	CURSED_TITLELINE,
 	CURSED_DIR,
 	CURSED_ERROR,
-	CURSED_INFO,
 
+	CURSED_INFO,
 	CURSED_TRACKWIN_ALBUM,
 
 	NR_CURSED
@@ -193,11 +195,12 @@ static unsigned char cursed_to_bg_idx[NR_CURSED] = {
 	COLOR_CMDLINE_BG,
 	COLOR_STATUSLINE_BG,
 
+	COLOR_STATUSLINE_PROGRESS_BG,
 	COLOR_TITLELINE_BG,
 	COLOR_WIN_BG,
 	COLOR_CMDLINE_BG,
-	COLOR_CMDLINE_BG,
 
+	COLOR_CMDLINE_BG,
 	COLOR_TRACKWIN_ALBUM_BG,
 };
 
@@ -217,11 +220,12 @@ static unsigned char cursed_to_fg_idx[NR_CURSED] = {
 	COLOR_CMDLINE_FG,
 	COLOR_STATUSLINE_FG,
 
+	COLOR_STATUSLINE_PROGRESS_FG,
 	COLOR_TITLELINE_FG,
 	COLOR_WIN_DIR,
 	COLOR_ERROR,
-	COLOR_INFO,
 
+	COLOR_INFO,
 	COLOR_TRACKWIN_ALBUM_FG,
 };
 
@@ -241,11 +245,12 @@ static unsigned char cursed_to_attr_idx[NR_CURSED] = {
 	COLOR_CMDLINE_ATTR,
 	COLOR_STATUSLINE_ATTR,
 
+	COLOR_STATUSLINE_PROGRESS_ATTR,
 	COLOR_TITLELINE_ATTR,
 	COLOR_WIN_ATTR,
 	COLOR_CMDLINE_ATTR,
-	COLOR_CMDLINE_ATTR,
 
+	COLOR_CMDLINE_ATTR,
 	COLOR_TRACKWIN_ALBUM_ATTR,
 };
 
@@ -257,6 +262,7 @@ enum {
 	TF_ARTIST,
 	TF_ALBUM,
 	TF_DISC,
+	TF_TOTAL_DISCS,
 	TF_TRACK,
 	TF_TITLE,
 	TF_PLAY_COUNT,
@@ -306,6 +312,7 @@ enum {
 	TF_SHUFFLE,
 	TF_PLAYLISTMODE,
 	TF_BPM,
+	TF_PANEL,
 
 	NR_TFS
 };
@@ -315,6 +322,7 @@ static struct format_option track_fopts[NR_TFS + 1] = {
 	DEF_FO_STR('a', "artist", 0),
 	DEF_FO_STR('l', "album", 0),
 	DEF_FO_INT('D', "discnumber", 1),
+	DEF_FO_INT('T', "totaldiscs", 1),
 	DEF_FO_INT('n', "tracknumber", 1),
 	DEF_FO_STR('t', "title", 0),
 	DEF_FO_INT('X', "play_count", 0),
@@ -364,6 +372,7 @@ static struct format_option track_fopts[NR_TFS + 1] = {
 	DEF_FO_STR('\0', "shuffle", 0),
 	DEF_FO_STR('\0', "playlist_mode", 0),
 	DEF_FO_INT('\0', "bpm", 0),
+	DEF_FO_INT('\0', "panel", 0),
 	DEF_FO_END
 };
 
@@ -450,44 +459,38 @@ fallback:
 
 /* screen updates {{{ */
 
-static void dump_print_buffer(int row, int col)
+static void dump_print_buffer_no_clear(int row, int col, size_t offset)
 {
 	if (using_utf8) {
-		(void) mvaddstr(row, col, print_buffer);
+		(void) mvaddstr(row, col, print_buffer.buffer + offset);
 	} else {
-		utf8_decode(print_buffer);
+		utf8_decode(print_buffer.buffer + offset);
 		(void) mvaddstr(row, col, conv_buffer);
 	}
 }
 
+static void dump_print_buffer(int row, int col)
+{
+	dump_print_buffer_no_clear(row, col, 0);
+	gbuf_clear(&print_buffer);
+}
+
 /* print @str into @buf
  *
  * if @str is shorter than @width pad with spaces
  * if @str is wider than @width truncate and add "..."
  */
-static int format_str(char *buf, const char *str, int width)
+static void format_str(struct gbuf *buf, const char *str, int width)
 {
-	int w = width;
-	size_t copy_bytes = u_copy_chars(buf, str, &w);
-
-	if (w > 0) {
-		memset(buf + copy_bytes, ' ', w);
-		copy_bytes += w;
-	} else if (str[copy_bytes] != '\0') {
-		copy_bytes = mark_clipped_text(buf, width);
-	}
-	buf[copy_bytes] = '\0';
-	return copy_bytes;
+	gbuf_add_ustr(buf, str, &width);
+	gbuf_set(buf, ' ', width);
 }
 
 static void sprint(int row, int col, const char *str, int width)
 {
-	int pos = 0;
-
-	print_buffer[pos++] = ' ';
-	pos += format_str(print_buffer + pos, str, width - 2);
-	print_buffer[pos++] = ' ';
-	print_buffer[pos] = 0;
+	gbuf_add_ch(&print_buffer, ' ');
+	format_str(&print_buffer, str, width - 2);
+	gbuf_add_ch(&print_buffer, ' ');
 	dump_print_buffer(row, col);
 }
 
@@ -539,6 +542,7 @@ static void fill_track_fopts_track_info(struct track_info *info)
 	fopt_set_str(&track_fopts[TF_ALBUM], info->album);
 	fopt_set_int(&track_fopts[TF_PLAY_COUNT], info->play_count, 0);
 	fopt_set_int(&track_fopts[TF_DISC], info->discnumber, info->discnumber == -1);
+	fopt_set_int(&track_fopts[TF_TOTAL_DISCS], info->totaldiscs, info->totaldiscs == -1);
 	fopt_set_int(&track_fopts[TF_TRACK], info->tracknumber, info->tracknumber == -1);
 	fopt_set_str(&track_fopts[TF_TITLE], info->title);
 	fopt_set_int(&track_fopts[TF_YEAR], info->date / 10000, info->date <= 0);
@@ -565,8 +569,8 @@ static void fill_track_fopts_track_info(struct track_info *info)
 	fopt_set_str(&track_fopts[TF_PUBLISHER], keyvals_get_val(info->comments, "publisher"));
 	fopt_set_str(&track_fopts[TF_WORK], keyvals_get_val(info->comments, "work"));
 	fopt_set_str(&track_fopts[TF_OPUS], keyvals_get_val(info->comments, "opus"));
-	fopt_set_str(&track_fopts[TF_PARTNUMBER], keyvals_get_val(info->comments, "partnumber"));
-	fopt_set_str(&track_fopts[TF_PART], keyvals_get_val(info->comments, "part"));
+	fopt_set_str(&track_fopts[TF_PARTNUMBER], keyvals_get_val(info->comments, "discnumber"));
+	fopt_set_str(&track_fopts[TF_PART], keyvals_get_val(info->comments, "discnumber"));
 	fopt_set_str(&track_fopts[TF_SUBTITLE], keyvals_get_val(info->comments, "subtitle"));
 	fopt_set_str(&track_fopts[TF_MEDIA], info->media);
 	fopt_set_int(&track_fopts[TF_VA], 0, !track_is_compilation(info->comments));
@@ -585,7 +589,20 @@ static int get_album_length(struct album *album)
 	int duration = 0;
 
 	rb_for_each_entry(track, tmp, &album->track_root, tree_node) {
-		duration += tree_track_info(track)->duration;
+		duration += max_i(0, tree_track_info(track)->duration);
+	}
+
+	return duration;
+}
+
+static int get_artist_length(struct artist *artist)
+{
+	struct album *album;
+	struct rb_node *tmp;
+	int duration = 0;
+
+	rb_for_each_entry(album, tmp, &artist->album_root, tree_node) {
+		duration += get_album_length(album);
 	}
 
 	return duration;
@@ -598,7 +615,9 @@ static void fill_track_fopts_album(struct album *album)
 	fopt_set_str(&track_fopts[TF_ALBUMARTIST], album->artist->name);
 	fopt_set_str(&track_fopts[TF_ARTIST], album->artist->name);
 	fopt_set_str(&track_fopts[TF_ALBUM], album->name);
-	fopt_set_time(&track_fopts[TF_ALBUMDURATION], get_album_length(album), 0);
+	int duration = get_album_length(album);
+	fopt_set_time(&track_fopts[TF_DURATION], duration, 0);
+	fopt_set_time(&track_fopts[TF_ALBUMDURATION], duration, 0);
 }
 
 static void fill_track_fopts_artist(struct artist *artist)
@@ -606,6 +625,7 @@ static void fill_track_fopts_artist(struct artist *artist)
 	const char *name = display_artist_sort_name ? artist_sort_name(artist) : artist->name;
 	fopt_set_str(&track_fopts[TF_ARTIST], name);
 	fopt_set_str(&track_fopts[TF_ALBUMARTIST], name);
+	fopt_set_time(&track_fopts[TF_DURATION], get_artist_length(artist), 0);
 }
 
 const struct format_option *get_global_fopts(void)
@@ -621,8 +641,12 @@ const struct format_option *get_global_fopts(void)
 	int buffer_fill, vol, vol_left, vol_right;
 	int duration = -1;
 
-	fopt_set_time(&track_fopts[TF_TOTAL], play_library ? lib_editable.total_time :
-			pl_playing_total_time(), 0);
+	unsigned int total_time = pl_playing_total_time();
+	if (cmus_queue_active())
+		total_time = play_queue_total_time();
+	else if (play_library)
+		total_time = lib_editable.total_time;
+	fopt_set_time(&track_fopts[TF_TOTAL], total_time, 0);
 
 	fopt_set_str(&track_fopts[TF_FOLLOW], follow_strs[follow]);
 	fopt_set_str(&track_fopts[TF_REPEAT], repeat_strs[repeat]);
@@ -691,13 +715,13 @@ static void print_tree(struct window *win, int row, struct iter *iter)
 		cursor_y = 1 + row;
 	}
 
-	print_buffer[0] = ' ';
+	gbuf_add_ch(&print_buffer, ' ');
 	if (album) {
 		fill_track_fopts_album(album);
-		format_print(print_buffer + 1, tree_win_w - 1, tree_win_format, track_fopts);
+		format_print(&print_buffer, tree_win_w - 1, tree_win_format, track_fopts);
 	} else {
 		fill_track_fopts_artist(artist);
-		format_print(print_buffer + 1, tree_win_w - 1, tree_win_artist_format, track_fopts);
+		format_print(&print_buffer, tree_win_w - 1, tree_win_artist_format, track_fopts);
 	}
 	dump_print_buffer(row + 1, tree_win_x);
 }
@@ -722,7 +746,7 @@ static void print_track(struct window *win, int row, struct iter *iter)
 
 		fill_track_fopts_album(album);
 
-		len = format_print(print_buffer, track_win_w, track_win_album_format, track_fopts);
+		len = format_print(&print_buffer, track_win_w, track_win_album_format, track_fopts);
 		dump_print_buffer(row + 1, track_win_x);
 
 		bkgdset(pairs[CURSED_SEPARATOR]);
@@ -753,7 +777,7 @@ static void print_track(struct window *win, int row, struct iter *iter)
 	} else if (*track_win_alt_format) {
 		format = track_win_alt_format;
 	}
-	format_print(print_buffer, track_win_w, format, track_fopts);
+	format_print(&print_buffer, track_win_w, format, track_fopts);
 	dump_print_buffer(row + 1, track_win_x);
 }
 
@@ -794,7 +818,7 @@ static void print_editable(struct window *win, int row, struct iter *iter)
 	} else if (*list_win_alt_format) {
 		format = list_win_alt_format;
 	}
-	format_print(print_buffer, win_w, format, track_fopts);
+	format_print(&print_buffer, win_w, format, track_fopts);
 	dump_print_buffer(row + 1, win_x);
 }
 
@@ -840,7 +864,7 @@ static void print_filter(struct window *win, int row, struct iter *iter)
 	/* is the filter currently active? */
 	int current = !!e->act_stat;
 	const char stat_chars[3] = " *!";
-	int ch1, ch2, ch3, pos;
+	int ch1, ch2, ch3;
 	const char *e_filter;
 
 	window_get_sel(win, &sel);
@@ -867,9 +891,8 @@ static void print_filter(struct window *win, int row, struct iter *iter)
 	}
 
 	snprintf(buf, sizeof(buf), "%c%c%c%-15s  %.235s", ch1, ch2, ch3, e->name, e_filter);
-	pos = format_str(print_buffer, buf, win_w - 1);
-	print_buffer[pos++] = ' ';
-	print_buffer[pos] = 0;
+	format_str(&print_buffer, buf, win_w - 1);
+	gbuf_add_ch(&print_buffer, ' ');
 	dump_print_buffer(row + 1, 0);
 }
 
@@ -877,7 +900,6 @@ static void print_help(struct window *win, int row, struct iter *iter)
 {
 	struct iter sel;
 	int selected;
-	int pos;
 	int active = 1;
 	char buf[OPTION_MAX_SIZE];
 	const struct help_entry *e = iter_to_help_entry(iter);
@@ -912,9 +934,8 @@ static void print_help(struct window *win, int row, struct iter *iter)
 		opt->get(opt->data, buf + len, sizeof(buf) - len);
 		break;
 	}
-	pos = format_str(print_buffer, buf, win_w - 1);
-	print_buffer[pos++] = ' ';
-	print_buffer[pos] = 0;
+	format_str(&print_buffer, buf, win_w - 1);
+	gbuf_add_ch(&print_buffer, ' ');
 	dump_print_buffer(row + 1, 0);
 }
 
@@ -928,8 +949,7 @@ static void update_window(struct window *win, int x, int y, int w, const char *t
 	win->changed = 0;
 
 	bkgdset(pairs[CURSED_WIN_TITLE]);
-	snprintf(print_buffer, sizeof(print_buffer), " %s ", title);
-	dump_print_buffer(y, x);
+	sprint(y, x, title, w);
 
 	nr_rows = window_get_nr_rows(win);
 	i = 0;
@@ -943,29 +963,47 @@ static void update_window(struct window *win, int x, int y, int w, const char *t
 	}
 
 	bkgdset(pairs[0]);
-	memset(print_buffer, ' ', w);
-	print_buffer[w] = 0;
+	gbuf_set(&print_buffer, ' ', w);
 	while (i < nr_rows) {
-		dump_print_buffer(y + i + 1, x);
+		dump_print_buffer_no_clear(y + i + 1, x, 0);
 		i++;
 	}
+	gbuf_clear(&print_buffer);
 }
 
 static void update_tree_window(void)
 {
-	char title[PRINT_BUFFER_SZ];
-	format_str(title, "Artist / Album", tree_win_w - 1);
-	update_window(lib_tree_win, tree_win_x, 0, tree_win_w, title, print_tree);
+	static GBUF(buf);
+	gbuf_clear(&buf);
+
+	gbuf_add_str(&buf, "Library");
+	if (worker_has_job())
+		gbuf_addf(&buf, " - %d tracks", lib_editable.nr_tracks);
+	update_window(lib_tree_win, tree_win_x, 0, tree_win_w + 1, buf.buffer, print_tree);
 }
 
 static void update_track_window(void)
 {
-	char title[PRINT_BUFFER_SZ];
+	static GBUF(title);
+	gbuf_clear(&title);
 
-	/* it doesn't matter what format options we use because the format
-	 * string does not contain any format charaters */
-	format_print(title, track_win_w - 2, "Track%= Library", track_fopts);
-	update_window(lib_track_win, track_win_x, 0, track_win_w, title,
+	struct iter iter;
+	struct album *album;
+	struct artist *artist;
+
+	const char *format_str = "Empty (use :add)";
+
+	if (window_get_sel(lib_tree_win, &iter)) {
+		if ((album = iter_to_album(&iter))) {
+			fill_track_fopts_album(album);
+			format_str = heading_album_format;
+		} else if ((artist = iter_to_artist(&iter))) {
+			fill_track_fopts_artist(artist);
+			format_str = heading_artist_format;
+		}
+	}
+	format_print(&title, track_win_w - 2, format_str, track_fopts);
+	update_window(lib_track_win, track_win_x, 0, track_win_w, title.buffer,
 			print_track);
 }
 
@@ -981,133 +1019,15 @@ static void print_pl_list(struct window *win, int row, struct iter *iter)
 	if (info.marked)
 		prefix = " * ";
 	size_t prefix_w = strlen(prefix);
-	format_str(print_buffer, prefix, prefix_w);
+	format_str(&print_buffer, prefix, prefix_w);
 
 	if (tree_win_w > prefix_w)
-		format_str(print_buffer + prefix_w, info.name,
+		format_str(&print_buffer, info.name,
 				tree_win_w - prefix_w);
 
 	dump_print_buffer(row + 1, 0);
 }
 
-static void update_pl_list(struct window *win)
-{
-	char title[PRINT_BUFFER_SZ];
-	format_str(title, "Playlist", tree_win_w - 1);
-	update_window(win, tree_win_x, 0, tree_win_w, title, print_pl_list);
-}
-
-static void update_pl_tracks(struct window *win)
-{
-	char title[PRINT_BUFFER_SZ];
-	int win_w_tmp = win_w;
-
-	win_x = track_win_x;
-	win_w = track_win_w;
-	win_active = pl_get_cursor_in_track_window();
-
-	get_global_fopts();
-	fopt_set_time(&track_fopts[TF_TOTAL], pl_visible_total_time(), 0);
-
-	format_print(title, track_win_w - 2, "Track%= %{total}", track_fopts);
-	update_window(win, track_win_x, 0, track_win_w, title, print_editable);
-
-	win_active = 1;
-	win_x = 0;
-	win_w = win_w_tmp;
-}
-
-static const char *pretty(const char *path)
-{
-	static int home_len = -1;
-	static char buf[PRINT_BUFFER_SZ];
-
-	if (home_len == -1)
-		home_len = strlen(home_dir);
-
-	if (strncmp(path, home_dir, home_len) || path[home_len] != '/')
-		return path;
-
-	buf[0] = '~';
-	strcpy(buf + 1, path + home_len);
-	return buf;
-}
-
-static const char * const sorted_names[2] = { "", "sorted by " };
-
-static void update_editable_window(struct editable *e, const char *title, const char *filename)
-{
-	char buf[PRINT_BUFFER_SZ];
-	int pos;
-
-	if (filename) {
-		if (using_utf8) {
-			/* already UTF-8 */
-		} else {
-			utf8_encode_to_buf(filename);
-			filename = conv_buffer;
-		}
-		snprintf(buf, sizeof(buf), "%s %.256s - %d tracks", title, pretty(filename), e->nr_tracks);
-	} else {
-		snprintf(buf, sizeof(buf), "%s - %d tracks", title, e->nr_tracks);
-	}
-
-	if (e->nr_marked) {
-		pos = strlen(buf);
-		snprintf(buf + pos, sizeof(buf) - pos, " (%d marked)", e->nr_marked);
-	}
-	pos = strlen(buf);
-	snprintf(buf + pos, sizeof(buf) - pos, " %s%s",
-			sorted_names[e->shared->sort_str[0] != 0],
-			e->shared->sort_str);
-
-	format_str(buf, buf, win_w - 2);
-	update_window(e->shared->win, 0, 0, win_w, buf, &print_editable);
-}
-
-static void update_sorted_window(void)
-{
-	current_track = (struct simple_track *)lib_cur_track;
-	update_editable_window(&lib_editable, "Library", lib_filename);
-}
-
-static void update_play_queue_window(void)
-{
-	current_track = NULL;
-	update_editable_window(&pq_editable, "Play Queue", NULL);
-}
-
-static void update_browser_window(void)
-{
-	char title[PRINT_BUFFER_SZ];
-	char *dirname;
-
-	if (using_utf8) {
-		/* already UTF-8 */
-		dirname = browser_dir;
-	} else {
-		utf8_encode_to_buf(browser_dir);
-		dirname = conv_buffer;
-	}
-	snprintf(title, sizeof(title), "Browser - %.501s", dirname);
-	format_str(title, title, win_w - 2);
-	update_window(browser_win, 0, 0, win_w, title, print_browser);
-}
-
-static void update_filters_window(void)
-{
-	char title[PRINT_BUFFER_SZ];
-	format_str(title, "Library Filters", win_w - 2);
-	update_window(filters_win, 0, 0, win_w, title, print_filter);
-}
-
-static void update_help_window(void)
-{
-	char title[PRINT_BUFFER_SZ];
-	format_str(title, "Settings", win_w - 2);
-	update_window(help_win, 0, 0, win_w, title, print_help);
-}
-
 static void draw_separator(void)
 {
 	int row;
@@ -1119,15 +1039,142 @@ static void draw_separator(void)
 		(void) mvaddch(row, tree_win_w, ACS_VLINE);
 }
 
+static void update_pl_list(struct window *win)
+{
+	if (pl_show_panel()) {
+		update_window(win, tree_win_x, 0, tree_win_w + 1, "Playlist", print_pl_list);
+		draw_separator();
+	}
+}
+
+static void update_pl_tracks(struct window *win)
+{
+	static GBUF(title);
+	gbuf_clear(&title);
+	int win_w_tmp = win_w;
+
+	if (pl_show_panel()) {
+		win_x = track_win_x;
+		win_w = track_win_w;
+	} else {
+		win_x = 0;
+		win_w = tree_win_w + 1 + track_win_w;
+	}
+	win_active = pl_get_cursor_in_track_window();
+
+	get_global_fopts();
+	fopt_set_int(&track_fopts[TF_PANEL], 1, !pl_show_panel());
+	fopt_set_str(&track_fopts[TF_TITLE], pl_visible_get_name());
+	fopt_set_time(&track_fopts[TF_DURATION], pl_visible_total_time(), 0);
+
+	format_print(&title, win_w - 2, heading_playlist_format, track_fopts);
+	update_window(win, win_x, 0, win_w, title.buffer, print_editable);
+
+	win_active = 1;
+	win_x = 0;
+	win_w = win_w_tmp;
+}
+
+static const char *pretty_path(const char *path)
+{
+	static int home_len = -1;
+	static GBUF(buf);
+
+	if (home_len == -1)
+		home_len = strlen(home_dir);
+
+	if (strncmp(path, home_dir, home_len) || path[home_len] != '/')
+		return path;
+
+	gbuf_clear(&buf);
+	gbuf_add_ch(&buf, '~');
+	gbuf_add_str(&buf, path + home_len);
+	return buf.buffer;
+}
+
+static const char * const sorted_names[2] = { "", "sorted by " };
+
+static void update_editable_window(struct editable *e, const char *title, const char *filename)
+{
+	static GBUF(buf);
+	gbuf_clear(&buf);
+
+	if (filename) {
+		if (using_utf8) {
+			/* already UTF-8 */
+		} else {
+			utf8_encode_to_buf(filename);
+			filename = conv_buffer;
+		}
+		gbuf_addf(&buf, "%s %.256s - %d tracks", title, pretty_path(filename), e->nr_tracks);
+	} else {
+		gbuf_addf(&buf, "%s - %d tracks", title, e->nr_tracks);
+	}
+
+	fopt_set_time(&track_fopts[TF_TOTAL], e->total_time, 0);
+	format_print(&buf, 0, " (%{total})", track_fopts);
+
+	if (e->nr_marked) {
+		gbuf_addf(&buf, " (%d marked)", e->nr_marked);
+	}
+	gbuf_addf(&buf, " %s%s",
+			sorted_names[e->shared->sort_str[0] != 0],
+			e->shared->sort_str);
+
+	update_window(e->shared->win, 0, 0, win_w, buf.buffer, &print_editable);
+}
+
+static void update_sorted_window(void)
+{
+	current_track = (struct simple_track *)lib_cur_track;
+	update_editable_window(&lib_editable, "Library", NULL);
+}
+
+static void update_play_queue_window(void)
+{
+	current_track = NULL;
+	update_editable_window(&pq_editable, "Play Queue", NULL);
+}
+
+static void update_browser_window(void)
+{
+	static GBUF(title);
+	gbuf_clear(&title);
+	char *dirname;
+
+	if (using_utf8) {
+		/* already UTF-8 */
+		dirname = browser_dir;
+	} else {
+		utf8_encode_to_buf(browser_dir);
+		dirname = conv_buffer;
+	}
+	gbuf_add_str(&title, "Browser - ");
+	gbuf_add_str(&title, dirname);
+	update_window(browser_win, 0, 0, win_w, title.buffer, print_browser);
+}
+
+static void update_filters_window(void)
+{
+	update_window(filters_win, 0, 0, win_w, "Library Filters", print_filter);
+}
+
+static void update_help_window(void)
+{
+	update_window(help_win, 0, 0, win_w, "Settings", print_help);
+}
+
 static void update_pl_view(int full)
 {
 	current_track = pl_get_playing_track();
 	pl_draw(update_pl_list, update_pl_tracks, full);
-	draw_separator();
 }
 
 static void do_update_view(int full)
 {
+	if (!ui_initialized)
+		return;
+
 	cursor_x = -1;
 	cursor_y = -1;
 
@@ -1164,9 +1211,57 @@ static void do_update_view(int full)
 
 static void do_update_statusline(void)
 {
-	format_print(print_buffer, win_w, statusline_format, get_global_fopts());
+	struct fp_len len;
+	len = format_print(&print_buffer, win_w, statusline_format, get_global_fopts());
 	bkgdset(pairs[CURSED_STATUSLINE]);
-	dump_print_buffer(LINES - 2, 0);
+	dump_print_buffer_no_clear(LINES - 2, 0, 0);
+
+	if (progress_bar && player_info.ti) {
+		int duration = player_info.ti->duration;
+		if (duration && duration >= player_info.pos) {
+			if (progress_bar == PROGRESS_BAR_LINE || progress_bar == PROGRESS_BAR_SHUTTLE) {
+				/* Draw a bar or short position marker within the blank space */
+				int shuttle_len = (progress_bar == PROGRESS_BAR_SHUTTLE) ? 2 : 0;
+				int bar_start = len.llen + len.mlen;
+				int bar_space = win_w - len.rlen - bar_start - shuttle_len;
+				if (bar_space >= 5) {
+					int bar_len = bar_space * player_info.pos / duration;
+				        if (progress_bar == PROGRESS_BAR_SHUTTLE) {
+						bar_start += bar_len;
+						bar_len = shuttle_len;
+					}
+					for (int x = bar_start; bar_len; --bar_len)
+						(void) mvaddstr(LINES - 2, x++, using_utf8 ? "━" : "-");
+				}
+			} else if (progress_bar == PROGRESS_BAR_COLOR) {
+				/* Draw over the played portion of bar in alt color */
+				int w = win_w * player_info.pos / duration;
+
+				int skip = w;
+				int buf_index = u_skip_chars(print_buffer.buffer, &skip, false);
+				print_buffer.buffer[buf_index] = '\0';
+
+				bkgdset(pairs[CURSED_STATUSLINE_PROGRESS]);
+				dump_print_buffer_no_clear(LINES - 2, 0, 0);
+
+			} else { // PROGRESS_BAR_COLOR_SHUTTLE
+				/* Redraw a few cols in alt color to mark the current position */
+				int shuttle_len = min_u(6, win_w);
+				int x = (win_w - shuttle_len) * player_info.pos / duration;
+
+				int skip = x;
+				int buf_index = u_skip_chars(print_buffer.buffer, &skip, false);
+
+				int end_offset = u_skip_chars(print_buffer.buffer + buf_index, &shuttle_len, true);
+				print_buffer.buffer[buf_index+end_offset] = '\0';
+
+				bkgdset(pairs[CURSED_STATUSLINE_PROGRESS]);
+				dump_print_buffer_no_clear(LINES - 2, x, buf_index);
+			}
+		}
+	}
+
+	gbuf_clear(&print_buffer);
 
 	if (player_info.error_msg)
 		error_msg("%s", player_info.error_msg);
@@ -1185,18 +1280,17 @@ static void dump_buffer(const char *buffer)
 static void do_update_commandline(void)
 {
 	char *str;
-	int w;
-	size_t idx;
+	size_t idx = 0;
 	char ch;
 
 	move(LINES - 1, 0);
-	if (error_buf[0]) {
+	if (error_buf.len != 0) {
 		if (msg_is_error) {
 			bkgdset(pairs[CURSED_ERROR]);
 		} else {
 			bkgdset(pairs[CURSED_INFO]);
 		}
-		addstr(error_buf);
+		addstr(error_buf.buffer);
 		clrtoeol();
 		return;
 	}
@@ -1226,60 +1320,38 @@ static void do_update_commandline(void)
 	}
 
 	/* COMMAND_MODE or SEARCH_MODE */
-	w = u_str_width(str);
 	ch = ':';
 	if (input_mode == SEARCH_MODE)
 		ch = search_direction == SEARCH_FORWARD ? '/' : '?';
 
-	if (w <= win_w - 2) {
+	int width = win_w - 2; // ':' at start and ' ' at end
+
+	/* width of the text in the buffer before and after cursor */
+	int cw = u_str_nwidth(str, cmdline.cpos);
+	int extra_w = u_str_width(str + cmdline.bpos);
+
+	/* shift by third of bar width to provide visual context when editing */
+	int context_w = min_u(extra_w, win_w / 3);
+
+	int skip = cw + context_w - width;
+	if (skip <= 0) {
 		addch(ch);
-		idx = u_copy_chars(print_buffer, str, &w);
-		print_buffer[idx] = 0;
-		dump_buffer(print_buffer);
-		clrtoeol();
+		cmdline_cursor_x = 1 + cw;
 	} else {
-		/* keep cursor as far right as possible */
-		int skip, width, cw;
-
-		/* cursor pos (width, not chars. doesn't count the ':') */
-		cw = u_str_nwidth(str, cmdline.cpos);
-
-		skip = cw + 2 - win_w;
-		if (skip > 0) {
-			/* skip the ':' */
-			skip--;
-
-			/* skip rest (if any) */
-			idx = u_skip_chars(str, &skip, true);
-
-			width = win_w;
-			idx = u_copy_chars(print_buffer, str + idx, &width);
-			while (width > 0) {
-				/* cursor is at end of the buffer
-				 * print 1, 2 or 3 spaces
-				 *
-				 * To clarify:
-				 *
-				 * If the last _skipped_ character was double-width we may need
-				 * to print 2 spaces.
-				 *
-				 * If the last _skipped_ character was invalid UTF-8 we may need
-				 * to print 3 spaces.
-				 */
-				print_buffer[idx++] = ' ';
-				width--;
-			}
-			print_buffer[idx] = 0;
-			dump_buffer(print_buffer);
-		} else {
-			/* print ':' + win_w - 1 chars */
-			addch(ch);
-			width = win_w - 1;
-			idx = u_copy_chars(print_buffer, str, &width);
-			print_buffer[idx] = 0;
-			dump_buffer(print_buffer);
-		}
+		/* ':' will not be printed */
+		skip--;
+		width++;
+		idx = u_skip_chars(str, &skip, true);
+		gbuf_set(&print_buffer, ' ', -skip);
+		width += skip;
+		cmdline_cursor_x = win_w - 1 - context_w;
 	}
+	/* allow printing in ' ' space we kept at end, cursor isn't always there */
+	width++;
+	gbuf_add_ustr(&print_buffer, str + idx, &width);
+	dump_buffer(print_buffer.buffer);
+	gbuf_clear(&print_buffer);
+	clrtoeol();
 }
 
 static void set_title(const char *title)
@@ -1295,9 +1367,12 @@ static void set_title(const char *title)
 
 static void do_update_titleline(void)
 {
+	if (!ui_initialized)
+		return;
+
 	bkgdset(pairs[CURSED_TITLELINE]);
 	if (player_info.ti) {
-		int i, use_alt_format = 0;
+		int use_alt_format = 0;
 		char *wtitle;
 
 		fill_track_fopts_track_info(player_info.ti);
@@ -1319,35 +1394,30 @@ static void do_update_titleline(void)
 		}
 
 		if (use_alt_format && *current_alt_format) {
-			format_print(print_buffer, win_w, current_alt_format, track_fopts);
+			format_print(&print_buffer, win_w, current_alt_format, track_fopts);
 		} else {
-			format_print(print_buffer, win_w, current_format, track_fopts);
+			format_print(&print_buffer, win_w, current_format, track_fopts);
 		}
 		dump_print_buffer(LINES - 3, 0);
 
 		/* set window title */
 		if (use_alt_format && *window_title_alt_format) {
-			format_print(print_buffer, PRINT_BUFFER_MAX_WIDTH,
+			format_print(&print_buffer, 0,
 					window_title_alt_format, track_fopts);
 		} else {
-			format_print(print_buffer,  PRINT_BUFFER_MAX_WIDTH,
+			format_print(&print_buffer, 0,
 					window_title_format, track_fopts);
 		}
 
-		/* remove whitespace */
-		i = strlen(print_buffer) - 1;
-		while (i > 0 && print_buffer[i] == ' ')
-			i--;
-		print_buffer[i + 1] = 0;
-
 		if (using_utf8) {
-			wtitle = print_buffer;
+			wtitle = print_buffer.buffer;
 		} else {
-			utf8_decode(print_buffer);
+			utf8_decode(print_buffer.buffer);
 			wtitle = conv_buffer;
 		}
 
 		set_title(wtitle);
+		gbuf_clear(&print_buffer);
 	} else {
 		move(LINES - 3, 0);
 		clrtoeol();
@@ -1356,48 +1426,11 @@ static void do_update_titleline(void)
 	}
 }
 
-static int cmdline_cursor_column(void)
-{
-	char *str;
-	int cw, skip, s;
-
-	str = cmdline.line;
-	if (!using_utf8) {
-		/* see do_update_commandline */
-		utf8_encode_to_buf(cmdline.line);
-		str = conv_buffer;
-	}
-
-	/* width of the text in the buffer before cursor */
-	cw = u_str_nwidth(str, cmdline.cpos);
-
-	if (1 + cw < win_w) {
-		/* whole line is visible */
-		return 1 + cw;
-	}
-
-	/* beginning of cmdline is not visible */
-
-	/* check if the first visible char in cmdline would be halved
-	 * double-width character (or invalid byte <xx>) which is not possible.
-	 * we need to skip the whole character and move cursor to win_w - 2
-	 * column. */
-	skip = cw + 2 - win_w;
-
-	/* skip the ':' */
-	skip--;
-
-	/* skip rest */
-	s = skip;
-	u_skip_chars(str, &s, true);
-	return win_w - 1 + s;
-}
-
 static void post_update(void)
 {
 	/* refresh makes cursor visible at least for urxvt */
 	if (input_mode == COMMAND_MODE || input_mode == SEARCH_MODE) {
-		move(LINES - 1, cmdline_cursor_column());
+		move(LINES - 1, cmdline_cursor_x);
 		refresh();
 		curs_set(1);
 	} else {
@@ -1490,12 +1523,13 @@ void update_filterline(void)
 	if (cur_view != TREE_VIEW && cur_view != SORTED_VIEW)
 		return;
 	if (lib_live_filter) {
-		char buf[PRINT_BUFFER_SZ];
+		static GBUF(buf);
+		gbuf_clear(&buf);
 		int w;
 		bkgdset(pairs[CURSED_STATUSLINE]);
-		snprintf(buf, sizeof(buf), "filtered: %s", lib_live_filter);
-		w = clamp(strlen(buf) + 2, win_w/4, win_w/2);
-		sprint(LINES-4, win_w-w, buf, w);
+		gbuf_addf(&buf, "filtered: %s", lib_live_filter);
+		w = clamp(u_str_width(buf.buffer) + 2, win_w/4, win_w/2);
+		sprint(LINES-4, win_w-w, buf.buffer, w);
 	}
 }
 
@@ -1503,12 +1537,13 @@ void info_msg(const char *format, ...)
 {
 	va_list ap;
 
+	gbuf_clear(&error_buf);
 	va_start(ap, format);
-	vsnprintf(error_buf, sizeof(error_buf), format, ap);
+	gbuf_vaddf(&error_buf, format, ap);
 	va_end(ap);
 
 	if (client_fd != -1) {
-		write_all(client_fd, error_buf, strlen(error_buf));
+		write_all(client_fd, error_buf.buffer, error_buf.len);
 		write_all(client_fd, "\n", 1);
 	}
 
@@ -1521,14 +1556,15 @@ void error_msg(const char *format, ...)
 {
 	va_list ap;
 
-	strcpy(error_buf, "Error: ");
+	gbuf_clear(&error_buf);
+	gbuf_add_str(&error_buf, "Error: ");
 	va_start(ap, format);
-	vsnprintf(error_buf + 7, sizeof(error_buf) - 7, format, ap);
+	gbuf_vaddf(&error_buf, format, ap);
 	va_end(ap);
 
-	d_print("%s\n", error_buf);
+	d_print("%s\n", error_buf.buffer);
 	if (client_fd != -1) {
-		write_all(client_fd, error_buf, strlen(error_buf));
+		write_all(client_fd, error_buf.buffer, error_buf.len);
 		write_all(client_fd, "\n", 1);
 	}
 
@@ -1539,19 +1575,20 @@ void error_msg(const char *format, ...)
 		error_time = time(NULL);
 		update_commandline();
 	} else {
-		warn("%s\n", error_buf);
-		error_buf[0] = 0;
+		warn("%s\n", error_buf.buffer);
+		gbuf_clear(&error_buf);
 	}
 }
 
 enum ui_query_answer yes_no_query(const char *format, ...)
 {
-	char buffer[PRINT_BUFFER_SZ];
+	static GBUF(buffer);
+	gbuf_clear(&buffer);
 	va_list ap;
 	int ret = 0;
 
 	va_start(ap, format);
-	vsnprintf(buffer, sizeof(buffer), format, ap);
+	gbuf_vaddf(&buffer, format, ap);
 	va_end(ap);
 
 	move(LINES - 1, 0);
@@ -1562,7 +1599,7 @@ enum ui_query_answer yes_no_query(const char *format, ...)
 	 * encoded in same charset as LC_CTYPE).
 	 */
 
-	addstr(buffer);
+	addstr(buffer.buffer);
 	clrtoeol();
 	refresh();
 
@@ -1683,7 +1720,7 @@ void set_view(int view)
 
 void enter_command_mode(void)
 {
-	error_buf[0] = 0;
+	gbuf_clear(&error_buf);
 	error_time = 0;
 	input_mode = COMMAND_MODE;
 	update_commandline();
@@ -1691,7 +1728,7 @@ void enter_command_mode(void)
 
 void enter_search_mode(void)
 {
-	error_buf[0] = 0;
+	gbuf_clear(&error_buf);
 	error_time = 0;
 	input_mode = SEARCH_MODE;
 	search_direction = SEARCH_FORWARD;
@@ -1700,7 +1737,7 @@ void enter_search_mode(void)
 
 void enter_search_backward_mode(void)
 {
-	error_buf[0] = 0;
+	gbuf_clear(&error_buf);
 	error_time = 0;
 	input_mode = SEARCH_MODE;
 	search_direction = SEARCH_BACKWARD;
@@ -1739,9 +1776,9 @@ static void clear_error(void)
 	if (t - error_time < 2)
 		return;
 
-	if (error_buf[0]) {
+	if (error_buf.len != 0) {
 		error_time = 0;
-		error_buf[0] = 0;
+		gbuf_clear(&error_buf);
 		update_commandline();
 	}
 }
@@ -1896,11 +1933,9 @@ static void update_window_size(void)
 		resizeterm(lines, columns);
 #endif
 		w = COLS;
-		if (w > PRINT_BUFFER_MAX_WIDTH)
-			w = PRINT_BUFFER_MAX_WIDTH;
 		h = LINES - 3;
-		if (w < 3)
-			w = 3;
+		if (w < 4)
+			w = 4;
 		if (h < 2)
 			h = 2;
 		win_w = w;
@@ -1918,12 +1953,19 @@ static void update_window_size(void)
 
 static void update(void)
 {
+	static bool first_update = true;
 	int needs_view_update = 0;
 	int needs_title_update = 0;
 	int needs_status_update = 0;
 	int needs_command_update = 0;
 	int needs_spawn = 0;
 
+	if (first_update) {
+		needs_title_update = 1;
+		needs_command_update = 1;
+		first_update = false;
+	}
+
 	if (needs_to_resize) {
 		update_window_size();
 		needs_title_update = 1;
@@ -1973,7 +2015,9 @@ static void update(void)
 	}
 
 	/* total time changed? */
-	if (play_library) {
+	if (cmus_queue_active()) {
+		needs_status_update += queue_needs_redraw();
+	} else if (play_library) {
 		needs_status_update += lib_editable.shared->win->changed;
 		lib_editable.shared->win->changed = 0;
 	} else {
@@ -1996,13 +2040,18 @@ static void update(void)
 			do_update_commandline();
 		post_update();
 	}
+
+	/* Reset changed flags */
+	queue_post_update();
 }
 
 static void handle_ch(uchar ch)
 {
 	clear_error();
 	if (input_mode == NORMAL_MODE) {
-		normal_mode_ch(ch);
+		if (!block_key_paste || !in_bracketed_paste) {
+			normal_mode_ch(ch);
+		}
 	} else if (input_mode == COMMAND_MODE) {
 		command_mode_ch(ch);
 		update_commandline();
@@ -2012,6 +2061,45 @@ static void handle_ch(uchar ch)
 	}
 }
 
+static void handle_csi(void) {
+	// after ESC[ until 0x40-0x7E (@A–Z[\]^_`a–z{|}~)
+	// https://en.wikipedia.org/wiki/ANSI_escape_code#CSI_(Control_Sequence_Introducer)_sequences
+	// https://www.ecma-international.org/wp-content/uploads/ECMA-48_5th_edition_june_1991.pdf
+
+	int c;
+	int buf[16]; // buffer a reasonable length
+	size_t buf_n = 0;
+	int overflow = 0;
+
+	while (1) {
+		c = getch();
+		if (c == ERR || c == 0) {
+			return;
+		}
+		if (buf_n < sizeof(buf)/sizeof(*buf)) {
+			buf[buf_n++] = c;
+		} else {
+			overflow = 1;
+		}
+		if (c >= 0x40 && c <= 0x7E) {
+			break;
+		}
+	}
+
+	if (overflow) {
+		return;
+	}
+
+	if (buf_n == 4) {
+		// bracketed paste
+		// https://invisible-island.net/xterm/ctlseqs/ctlseqs.html#h2-Bracketed-Paste-Mode
+		if (buf[0] == '2' && buf[1] == '0' && (buf[2] == '0' || buf[2] == '1') && buf[3] == '~') {
+			in_bracketed_paste = buf[2] == '0';
+			return;
+		}
+	}
+}
+
 static void handle_escape(int c)
 {
 	clear_error();
@@ -2030,7 +2118,9 @@ static void handle_key(int key)
 {
 	clear_error();
 	if (input_mode == NORMAL_MODE) {
-		normal_mode_key(key);
+		if (!block_key_paste || !in_bracketed_paste) {
+			normal_mode_key(key);
+		}
 	} else if (input_mode == COMMAND_MODE) {
 		command_mode_key(key);
 		update_commandline();
@@ -2090,8 +2180,11 @@ static void u_getch(void)
 		cbreak();
 		int e_key = getch();
 		halfdelay(5);
-		if (e_key != ERR && e_key != 0) {
-			handle_escape(e_key);
+		if (e_key != ERR) {
+			if (e_key == '[')
+				handle_csi();
+			else if (e_key != 0)
+				handle_escape(e_key);
 			return;
 		}
 	}
@@ -2359,6 +2452,9 @@ static void init_all(void)
 	/* plugins have been loaded so we know what plugin options are available */
 	options_add();
 
+	/* cache the normalized env vars for pl_env */
+	pl_env_init();
+
 	lib_init();
 	searchable = tree_searchable;
 	cmus_init();
@@ -2372,12 +2468,13 @@ static void init_all(void)
 
 	/* almost everything must be initialized now */
 	options_load();
+	pl_init_options();
 	if (mpris)
 		mpris_init();
 
 	/* finally we can set the output plugin */
 	player_set_op(output_plugin);
-	if (!soft_vol)
+	if (!soft_vol || pause_on_output_change)
 		mixer_open();
 
 	lib_autosave_filename = xstrjoin(cmus_config_dir, "/lib.pl");
@@ -2397,6 +2494,10 @@ static void init_all(void)
 
 	init_curses();
 
+	// enable bracketed paste (will be ignored if not supported)
+	printf("\033[?2004h");
+	fflush(stdout);
+
 	if (resume_cmus) {
 		resume_load();
 		cmus_add(play_queue_append, play_queue_autosave_filename,
@@ -2415,6 +2516,10 @@ static void exit_all(void)
 {
 	endwin();
 
+	// disable bracketed paste
+	printf("\033[?2004l");
+	fflush(stdout);
+
 	if (resume_cmus)
 		resume_exit();
 	options_exit();
