@@ -1,6 +1,7 @@
-/* 
+/*
+ * Copyright 2008-2013 Various Authors
  * Copyright 2004 Timo Hirvonen
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -10,16 +11,17 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef _XSTRJOIN_H
-#define _XSTRJOIN_H
+#ifndef CMUS_XSTRJOIN_H
+#define CMUS_XSTRJOIN_H
 
-extern char *xstrjoin(const char *a, const char *b);
+#include "utils.h"
+
+char *xstrjoin_slice(struct slice);
+#define xstrjoin(...) xstrjoin_slice(TO_SLICE(const char *, __VA_ARGS__))
 
 #endif
